@@ -120,7 +120,7 @@ static struct bpf_pcap * add_pcap(char * iface, pcap_t * pcap)
  strncpy(bpc->iface, iface, sizeof(bpc->iface) - 1);
  bpc->iface[sizeof(bpc->iface) - 1] = '\0';
  pcap_lookupnet(iface, &network, &netmask, 0);
- pcap_compile(pcap, &bpc->bpf_filter, "", 0, netmask); 
+ pcap_compile(pcap, &bpc->bpf_filter, "", 1, netmask); 
  bpc->next = pcaps;
  pcaps = bpc;
  return bpc;
@@ -183,7 +183,7 @@ static pcap_t * new_pcap(char * iface)
    return NULL;
  }
  
- if(pcap_compile(pcap, &filter_prog, filter, 0, netmask) < 0)
+ if(pcap_compile(pcap, &filter_prog, filter, 1, netmask) < 0)
  {
   pcap_perror(pcap, "pcap_compile");
   pcap_close(pcap);
@@ -466,7 +466,8 @@ again:
 #endif       
        pcap_lookupnet(clients[i].iface, &network, &netmask, 0);
      /*  pcap_restart(NULL);    */    
-       if ( pcap_compile(pcap, &clients[i].bpf_filter, buf, 0, netmask) < 0 )
+       if ( pcap_compile(pcap, &clients[i].bpf_filter, buf, 1, netmask) 
+< 0 )
 	 {
 	 if ( pcap_compile_failed == 0 )
 		{
@@ -504,14 +505,14 @@ static int add_clients(int soc)
 {
  fd_set rd;
  struct timeval tv = {0,0};
- int clnt;
+ unsigned int clnt;
  
  FD_ZERO(&rd);
  FD_SET(soc, &rd);
  if(select(soc+1, &rd, &rd, &rd, &tv) > 0)
  {
   struct sockaddr_un soca;
-  int len = sizeof(soca);
+  unsigned int len = sizeof(soca);
   clnt = accept(soc, (struct sockaddr*)&soca,&len); 
   if(clnt > 0)
   {
@@ -950,7 +951,7 @@ int bpf_open_live(char * iface, char * filter)
    return -1;
  }
  
- if(pcap_compile(ret, &filter_prog, filter, 0, netmask) < 0)
+ if(pcap_compile(ret, &filter_prog, filter, 1, netmask) < 0)
  {
   pcap_perror(ret, "pcap_compile");
   pcap_close(ret);
