@@ -69,6 +69,7 @@ typedef struct {
 
   char*		buf;		/* NULL if unbuffered */
   int		bufsz, bufcnt, bufptr;
+  int 		last_err;
 } nessus_connection;
 
 /* 
@@ -152,6 +153,23 @@ static int data_left(soc)
 } /* data_left */
 #endif
 
+int
+stream_get_err(fd)
+ int fd;
+{
+ nessus_connection *p;
+ 
+ if(!NESSUS_STREAM(fd))
+    {
+     errno = EINVAL;
+     return -1;
+    }
+    
+    
+ p = &(connections[fd - NESSUS_FD_OFF]);
+ return p->last_err;
+}
+
 /*
  * Returns a free file descriptor
  */
@@ -175,6 +193,8 @@ get_connection_fd()
  return -1;
 }
 
+
+
 static int
 release_connection_fd(fd)
  int fd;
@@ -250,6 +270,7 @@ nessus_register_connection(s, ssl)
   p->port = 0;			/* just used for debug */
   p->fd = s;
   p->transport = (ssl != NULL) ? NESSUS_ENCAPS_SSLv23 : NESSUS_ENCAPS_IP;
+  p->last_err  = 0;
   return fd;
 }
 
@@ -607,7 +628,10 @@ open_SSL_connection(fp, timeout, cert, key, passwd, cert_names)
 	{
 	  d = tictac + timeout - time(NULL);
 	  if (d <= 0)
+	    {
+	    fp->last_err = ETIMEDOUT;
 	    return -1;
+            }
 	  to.tv_sec = d;
 	  to.tv_usec = 0;
 	  errno = 0;
@@ -620,7 +644,10 @@ open_SSL_connection(fp, timeout, cert, key, passwd, cert_names)
 	}
       while (ret < 0 && errno == EINTR);
       if (ret <= 0)
+	{
+	fp->last_err = ETIMEDOUT;
 	return -1;
+        }
     }
   /*NOTREACHED*/
 }
@@ -719,6 +746,7 @@ open_stream_connection(args, port, transport, timeout)
  fp->transport = transport;
  fp->timeout   = timeout;
  fp->port      = port;
+ fp->last_err  = 0;
  set_ids_evasion_mode(args, fp);
 
  if(fp->options & NESSUS_CNX_IDS_EVASION_FAKE_RST)
@@ -939,6 +967,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
       fp = &(connections[fd - NESSUS_FD_OFF]);
       trp = fp->transport;
       realfd = fp->fd;
+      fp->last_err = 0;
       if (fp->timeout != -2)
 	timeout = fp->timeout;
     }
@@ -989,11 +1018,17 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	      ret = recv(realfd, buf + total, max_len - total, waitall);
 	      if (ret < 0)
 		if (errno != EINTR)
+                  {
+		  fp->last_err = errno;
 		  return total;
+                  }
 		else
 		  ret = 0;
 	      else if (ret == 0) /* EOF */
+                {
+                fp->last_err = EPIPE;
 		return total;
+                }
 	      /*ret > 0*/
 	      total += ret; 
 	      if (min_len > 0 && total >= min_len)
@@ -1001,6 +1036,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	      flag = 0;
 	    }
 	}
+      if ( t >= timeout ) fp->last_err = ETIMEDOUT;
       return total;
     }
 
@@ -1025,7 +1061,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
       FD_SET(realfd, &fdr); FD_SET(realfd, &fdw); 
       now = then = time(NULL);
       for (t = 0; timeout <= 0 || t < timeout; t = now - then )
-	{
+	{	
           now = time(NULL);
 	  tv.tv_sec = INCR_TIMEOUT; tv.tv_usec = 0;
 	  select_status = select ( realfd + 1, &fdr, &fdw, NULL, &tv );
@@ -1072,12 +1108,14 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 #if DEBUG_SSL > 2
 		      fprintf(stderr, "SSL_Connect[%d]: SSL_ERROR_ZERO_RETURN\n", getpid());
 #endif
+		      fp->last_err = EPIPE;
 		      return total;
 
 		    default:
 #if DEBUG_SSL > 0
 		      sslerror2("SSL_read", err);
 #endif
+		      fp->last_err = EPIPE;
 		      return total;
 		    }
 		}
@@ -1094,6 +1132,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	  else if (total >= min_len)
 		return total;
 	}
+      if ( t >= timeout ) fp->last_err = ETIMEDOUT;
       return total;
 #endif
     default :
@@ -1205,6 +1244,7 @@ write_stream_connection4(fd, buf0, n, i_opt)
     }
 
  fp = &(connections[fd - NESSUS_FD_OFF]);
+ fp->last_err = 0;
  
 #if DEBUG_SSL > 8
  fprintf(stderr, "> write_stream_connection(%d, 0x%x, %d, 0x%x) \tE=%d 0=0x%x\n",
@@ -1229,7 +1269,11 @@ write_stream_connection4(fd, buf0, n, i_opt)
        ret = send(fp->fd, buf + count, n - count, i_opt);
 
     if(ret <= 0)
-      break;
+      {
+       if ( ret < 0 ) fp->last_err = errno;
+       else fp->last_err = EPIPE;
+       break;
+      }
      
      count += ret;
     }
@@ -1272,6 +1316,7 @@ write_stream_connection4(fd, buf0, n, i_opt)
 #if DEBUG_SSL > 0
 		  sslerror2("SSL_write", err);
 #endif      
+		  fp->last_err = EPIPE;
   	break;
      }
 	      if (fp->timeout >= 0)
@@ -1290,6 +1335,7 @@ write_stream_connection4(fd, buf0, n, i_opt)
 #if DEBUG_SSL > 0
 		  nessus_perror("select");
 #endif
+		  fp->last_err = ETIMEDOUT;
 		  break;
 		}
 	    }
@@ -2265,3 +2311,23 @@ error:
  *data_sz = 0;
  return -1;
 }
+
+
+ExtFunc int stream_pending(int fd)
+{
+  nessus_connection * fp;
+ if ( ! NESSUS_STREAM(fd) )
+ {
+  errno = EINVAL;
+  return -1;
+ }
+ fp = &(connections[fd - NESSUS_FD_OFF]);
+
+ if ( fp->bufcnt )
+        return fp->bufcnt;
+#ifdef HAVE_SSL
+ else if ( fp->transport != NESSUS_ENCAPS_IP )
+        return SSL_pending(fp->ssl);
+#endif
+ return 0;
+}
